import java.awt.Dimension;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.SpringLayout;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**This class contains the GUI and functionality of the simple simulator.
 * 
 * @author Andreas Carlsen
 *
 */
public class simpleSimulator extends JPanel {
	private static final long serialVersionUID = 1L;
	private JLabel gen1, gen2, gen3, gen4, lblgen1, lblgen2, lblgen3,lblgen4, production, lblproduction, 
	oc, lbloc, slidVal, lblUnbal, lblBal, bgen1, bgen2, bgen3, bgen4, lblbgen1, lblbgen2, lblbgen3,lblbgen4, 
	bproduction, lblbproduction, boc, lblboc;
	private JSlider slidProduction;
	double[][] SFOC_meas = new double[4][12];
	SFOCVector svec;
	double[] wmax= {18774,14075.999,14075.999,18774};
	double[] low = {25,25,25,25};
	double[] up = {85,85,85,85};
	int[] gen_pris = {0,2,1,3};
	unbalancedLoadSetPoints z;
	balancedLoadSetPoints e;
	double[] opt_load;
	double[] eq_load;
	generatorChooser chooser;
	double opt_oc, eq_oc;
	
	/**Constructor that initializes the variables and the GUI
	 * 
	 */
	public simpleSimulator() {
		
		SFOC_meas[0][0] = 226.3;
		SFOC_meas[0][1] = 199.8;
		SFOC_meas[0][2] = 190.6;
		SFOC_meas[0][3] = 189;
		SFOC_meas[0][4] = 185.8;
		SFOC_meas[0][5] = 188.8;
		SFOC_meas[0][6] = 184.3;
		SFOC_meas[0][7] = 181.5;
		SFOC_meas[0][8] = 179;
		SFOC_meas[0][9] = 182.5;
		SFOC_meas[0][10] = 183.8;
		SFOC_meas[0][11] = 186.8;
		SFOC_meas[1][0] = 226.4;
		SFOC_meas[1][1] = 205.64;
		SFOC_meas[1][2] = 191.8;
		SFOC_meas[1][3] = 190.672;
		SFOC_meas[1][4] = 188.744;
		SFOC_meas[1][5] = 188.416;
		SFOC_meas[1][6] = 187.1;
		SFOC_meas[1][7] = 183.75;
		SFOC_meas[1][8] = 180.4;
		SFOC_meas[1][9] = 183.45;
		SFOC_meas[1][10] = 186.5;
		SFOC_meas[1][11] = 188.2;	
		SFOC_meas[2][0] = 224.6;
		SFOC_meas[2][1] = 198;
		SFOC_meas[2][2] = 191.2;
		SFOC_meas[2][3] = 187.6;
		SFOC_meas[2][4] = 186.6;
		SFOC_meas[2][5] = 188.3;
		SFOC_meas[2][6] = 184.8;
		SFOC_meas[2][7] = 181.9;
		SFOC_meas[2][8] = 180.7;
		SFOC_meas[2][9] = 183.2;
		SFOC_meas[2][10] = 184.1;
		SFOC_meas[2][11] = 187.9;
		SFOC_meas[3][0] = 225;
		SFOC_meas[3][1] = 202.86;
		SFOC_meas[3][2] = 188.1;
		SFOC_meas[3][3] = 187.548;
		SFOC_meas[3][4] = 186.996;
		SFOC_meas[3][5] = 186.444;
		SFOC_meas[3][6] = 185.8;
		SFOC_meas[3][7] = 183.5;
		SFOC_meas[3][8] = 181.2;
		SFOC_meas[3][9] = 182.2;
		SFOC_meas[3][10] = 184.2;
		SFOC_meas[3][11] = 187.6;
		
		svec = new SFOCVector(SFOC_meas);
		
		z = new unbalancedLoadSetPoints(wmax, svec);
		e = new balancedLoadSetPoints(wmax, up);
		
		slidProduction = new JSlider(0, (int) (0.85*(wmax[0]+wmax[1]+wmax[2]+wmax[3])));
		slidProduction.setMinimum((int) (1+0.25*(wmax[this.gen_pris[0]])));
		slidProduction.addChangeListener(new sliderListener());
		
		String temp = Integer.toString(slidProduction.getValue());
		slidVal = new JLabel(temp + "kW");
		
		chooser = new generatorChooser(up);
		
		z.setGen_pris(gen_pris);
		z.setBin_var(chooser.setRunningGens(wmax,(double)slidProduction.getValue(), gen_pris));
		z.setHigh_limit(up);
		z.setLow_limit(low);
		z.setDemand((double)slidProduction.getValue());
		
		e.setDemand((double)slidProduction.getValue());
		
		opt_load = z.calculateOptimalLoad();
		eq_load = e.calculateOptimalLoad();
		
		opt_oc = svec.getValueOfIndex(opt_load[0], 0)*(opt_load[0])*z.getW_max()[0]/100000+svec.getValueOfIndex(opt_load[1], 1)*(opt_load[1])*z.getW_max()[1]/100000
				+svec.getValueOfIndex(opt_load[2], 2)*(opt_load[2])*z.getW_max()[2]/100000+svec.getValueOfIndex(opt_load[3], 3)*(opt_load[3])*z.getW_max()[3]/100000;
		
		eq_oc = svec.getValueOfIndex(eq_load[0], 0)*(eq_load[0])*e.getW_max()[0]/100000+svec.getValueOfIndex(eq_load[1], 1)*(eq_load[1])*e.getW_max()[1]/100000
				+svec.getValueOfIndex(eq_load[2], 2)*(eq_load[2])*e.getW_max()[2]/100000+svec.getValueOfIndex(eq_load[3], 3)*(eq_load[3])*e.getW_max()[3]/100000;
		
		if(eq_oc<opt_oc){
			opt_oc=eq_oc;
			opt_load=eq_load;
		}
		
		lblUnbal = new JLabel("UNBALANCED");
		lblBal = new JLabel("BALANCED");
		
		gen1 = new JLabel(Double.toString(opt_load[0]) + "%");
		gen2 = new JLabel(Double.toString(opt_load[1]) + "%");
		gen3 = new JLabel(Double.toString(opt_load[2]) + "%");
		gen4 = new JLabel(Double.toString(opt_load[3]) + "%");
		lblgen1 = new JLabel("Generator 1:");
		lblgen2 = new JLabel("Generator 2:");
		lblgen3 = new JLabel("Generator 3:");
		lblgen4 = new JLabel("Generator 4:");
		production = new JLabel(Double.toString(opt_load[0]*wmax[0]/100+opt_load[1]*wmax[1]/100+opt_load[2]*wmax[2]/100+opt_load[3]*wmax[3]/100) + " kW");
		lblproduction = new JLabel("Total production:");
		oc = new JLabel(Double.toString(opt_oc)+" kg");
		lbloc = new JLabel("Total oil consumption:");
		
		bgen1 = new JLabel(Double.toString(eq_load[0]) + "%");
		bgen2 = new JLabel(Double.toString(eq_load[1]) + "%");
		bgen3 = new JLabel(Double.toString(eq_load[2]) + "%");
		bgen4 = new JLabel(Double.toString(eq_load[3]) + "%");
		lblbgen1 = new JLabel("Generator 1 (16MW):");
		lblbgen2 = new JLabel("Generator 2 (8MW):");
		lblbgen3 = new JLabel("Generator 3 (16MW):");
		lblbgen4 = new JLabel("Generator 4 (8MW):");
		bproduction = new JLabel(Double.toString(eq_load[0]*wmax[0]/100+eq_load[1]*wmax[1]/100+eq_load[2]*wmax[2]/100+eq_load[3]*wmax[3]/100) + " kW");
		lblbproduction = new JLabel("Total production:");
		boc = new JLabel(Double.toString(eq_oc)+" kg");
		lblboc = new JLabel("Total oil consumption:");
		
		SpringLayout spring_layout = new SpringLayout();
		setLayout(spring_layout);
		add(lblUnbal);
		add(lblBal);
		add(lblgen1);
		add(lblgen2);
		add(lblgen3);
		add(lblgen4);
		add(gen1);
		add(gen2);
		add(gen3);
		add(gen4);
		add(production);
		add(lblproduction);
		add(oc);
		add(lbloc);
		add(lblbgen1);
		add(lblbgen2);
		add(lblbgen3);
		add(lblbgen4);
		add(bgen1);
		add(bgen2);
		add(bgen3);
		add(bgen4);
		add(bproduction);
		add(lblbproduction);
		add(boc);
		add(lblboc);
		add(slidProduction);
		add(slidVal);
		
		spring_layout.putConstraint(SpringLayout.WEST, lblUnbal,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblUnbal,5,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblBal,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblBal,5,SpringLayout.NORTH, this);
		
		spring_layout.putConstraint(SpringLayout.WEST, lblgen1,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblgen1,35,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblgen2,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblgen2,65,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblgen3,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblgen3,95,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblgen4,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblgen4,125,SpringLayout.NORTH, this);
    	
    	spring_layout.putConstraint(SpringLayout.WEST, gen1,140,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, gen1,35,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, gen2,140,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, gen2,65,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, gen3,140,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, gen3,95,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, gen4,140,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, gen4,125,SpringLayout.NORTH, this);
    	
    	spring_layout.putConstraint(SpringLayout.WEST, lblproduction,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblproduction,180,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, production,140,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, production,180,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lbloc,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lbloc,210,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, oc,140,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, oc,210,SpringLayout.NORTH, this);
    	


    	
    	spring_layout.putConstraint(SpringLayout.WEST, lblbgen1,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblbgen1,35,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblbgen2,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblbgen2,65,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblbgen3,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblbgen3,95,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblbgen4,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblbgen4,125,SpringLayout.NORTH, this);
    	
    	spring_layout.putConstraint(SpringLayout.WEST, bgen1,440,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, bgen1,35,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, bgen2,440,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, bgen2,65,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, bgen3,440,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, bgen3,95,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, bgen4,440,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, bgen4,125,SpringLayout.NORTH, this);
    	
    	spring_layout.putConstraint(SpringLayout.WEST, lblbproduction,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblbproduction,180,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, bproduction,440,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, bproduction,180,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, lblboc,305,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, lblboc,210,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, boc,440,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, boc,210,SpringLayout.NORTH, this);
    	
    	spring_layout.putConstraint(SpringLayout.WEST, slidProduction,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, slidProduction,240,SpringLayout.NORTH, this);
    	spring_layout.putConstraint(SpringLayout.WEST, slidVal,5,SpringLayout.WEST, this);
    	spring_layout.putConstraint(SpringLayout.NORTH, slidVal,270,SpringLayout.NORTH, this);
    	
    	this.setPreferredSize(new Dimension(600, 300));
	}
	
	/**This class listenes for changes in the power demand and updates the GUI accordingly.
	 * 
	 * @author Andreas Carlsen
	 *
	 */
	class sliderListener implements ChangeListener{

		@Override
		public void stateChanged(ChangeEvent arg0) {
			z.setBin_var(chooser.setRunningGens(wmax,(double)slidProduction.getValue(), gen_pris));
			z.setDemand((double)slidProduction.getValue());
			opt_load = z.calculateOptimalLoad();
			
			e.setDemand((double)slidProduction.getValue());
			eq_load = e.calculateOptimalLoad();
			
			opt_oc = svec.getValueOfIndex(opt_load[0], 0)*(opt_load[0])*z.getW_max()[0]/100000+svec.getValueOfIndex(opt_load[1], 1)*(opt_load[1])*z.getW_max()[1]/100000
					+svec.getValueOfIndex(opt_load[2], 2)*(opt_load[2])*z.getW_max()[2]/100000+svec.getValueOfIndex(opt_load[3], 3)*(opt_load[3])*z.getW_max()[3]/100000;
			
			eq_oc = svec.getValueOfIndex(eq_load[0], 0)*(eq_load[0])*e.getW_max()[0]/100000+svec.getValueOfIndex(eq_load[1], 1)*(eq_load[1])*e.getW_max()[1]/100000
					+svec.getValueOfIndex(eq_load[2], 2)*(eq_load[2])*e.getW_max()[2]/100000+svec.getValueOfIndex(eq_load[3], 3)*(eq_load[3])*e.getW_max()[3]/100000;
			
			gen1.setText(Double.toString(opt_load[0]) + "%");
			gen2.setText(Double.toString(opt_load[1]) + "%");
			gen3.setText(Double.toString(opt_load[2]) + "%");
			gen4.setText(Double.toString(opt_load[3]) + "%");
			slidVal.setText(Integer.toString(slidProduction.getValue()) + "kW");
			production.setText((Double.toString(opt_load[0]*wmax[0]/100+opt_load[1]*wmax[1]/100+opt_load[2]*wmax[2]/100+opt_load[3]*wmax[3]/100)) + " kW");
			oc.setText(Double.toString(opt_oc)+" kg");
			
			bgen1.setText(Double.toString(eq_load[0]) + "%");
			bgen2.setText(Double.toString(eq_load[1]) + "%");
			bgen3.setText(Double.toString(eq_load[2]) + "%");
			bgen4.setText(Double.toString(eq_load[3]) + "%");
			bproduction.setText((Double.toString(eq_load[0]*wmax[0]/100+eq_load[1]*wmax[1]/100+eq_load[2]*wmax[2]/100+eq_load[3]*wmax[3]/100)) + " kW");
			boc.setText(Double.toString(eq_oc)+" kg");
		}
		
	}
	
	/**Main method of the simple simulator.
	 */
	public static void main (String args[]){
		JPanel panel = new simpleSimulator();
		JFrame frame = new JFrame();
		frame.setContentPane(panel);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.pack();
		frame.setVisible(true);
	}

}
